@@ -44,10 +44,18 @@ class UnknownPermissionException(Exception):
     pass
 
 
-__all__ = ('register', 'grant', 'revoke', 'grant_group', 'revoke_group', \
-               'get_user_perms', 'get_group_perms', 'get_model_perms', \
-               'revoke_all', 'revoke_all_group', 'get_users', 'set_user_perms', \
-               'set_group_perms', 'get_groups', 'filter_on_perms')
+__all__ = (
+    'register',
+    'grant', 'grant_group',
+    'revoke', 'revoke_group',
+    'get_user_perms', 'get_group_perms',
+    'revoke_all', 'revoke_all_group',
+    'set_user_perms', 'set_group_perms',
+    'get_users', 'get_groups',
+    "user_has_any_perms", "group_has_any_perms",
+    'get_model_perms',
+    'filter_on_perms',
+)
 
 permission_map = {}
 """
@@ -60,6 +68,20 @@ permissions_for_model = {}
 A mapping of Models to lists of permissions defined for that model.
 """
 
+forbidden = set([
+    "full_clean",
+    "clean_fields",
+    "clean",
+    "validate_unique",
+    "save",
+    "pk",
+    "delete",
+    "get_absolute_url",
+])
+"""
+Names reserved by Django for Model instances.
+"""
+
 _DELAYED = []
 def register(perms, model):
     """
@@ -78,6 +100,10 @@ def register(perms, model):
         warn("Using a single permission is deprecated!")
         perms = [perms]
 
+    for perm in perms:
+        if perm in forbidden:
+            raise RegistrationException("Permission %s is a reserved name!")
+
     try:
         return _register(perms, model)
     except db.utils.DatabaseError:
@@ -165,9 +191,11 @@ def grant(user, perm, obj):
         raise UnknownPermissionException(perm)
 
     permissions = permission_map[model]
-    properties = dict(user=user, obj=obj)
 
-    user_perms, chaff = permissions.objects.get_or_create(**properties)
+    try:
+        user_perms = permissions.objects.get(user=user, obj=obj)
+    except permissions.DoesNotExist:
+        user_perms = permissions(user=user, obj=obj)
 
     # XXX could raise FieldDoesNotExist
     if not getattr(user_perms, perm):
@@ -187,9 +215,11 @@ def grant_group(group, perm, obj):
         raise UnknownPermissionException(perm)
     
     permissions = permission_map[model]
-    properties = dict(group=group, obj=obj)
 
-    group_perms, chaff = permissions.objects.get_or_create(**properties)
+    try:
+        group_perms = permissions.objects.get(group=group, obj=obj)
+    except permissions.DoesNotExist:
+        group_perms = permissions(group=group, obj=obj)
 
     # XXX could raise FieldDoesNotExist
     if not getattr(group_perms, perm):
@@ -202,25 +232,32 @@ def grant_group(group, perm, obj):
 def set_user_perms(user, perms, obj):
     """
     Set User permissions to exactly the specified permissions.
-    """
-
-    model = obj.__class__
-    permissions = permission_map[model]
-    all_perms = dict((p, False) for p in get_model_perms(model))
-    for perm in perms:
-        all_perms[perm] = True
-
-    user_perms, chaff = permissions.objects.get_or_create(user=user, obj=obj)
-
-    for perm, enabled in all_perms.iteritems():
-        if enabled and not getattr(user_perms, perm):
-            granted.send(sender=user, perm=perm, object=obj)
-        elif not enabled and getattr(user_perms, perm):
-            revoked.send(sender=user, perm=perm, object=obj)
-
-        setattr(user_perms, perm, enabled)
-
-    user_perms.save()
+    """    
+    if perms:
+        model = obj.__class__
+        permissions = permission_map[model]
+        
+        all_perms = dict((p, False) for p in get_model_perms(model))
+        for perm in perms:
+            all_perms[perm] = True
+        
+        try:
+            user_perms = permissions.objects.get(user=user, obj=obj)
+        except permissions.DoesNotExist:
+            user_perms = permissions(user=user, obj=obj)
+        
+        for perm, enabled in all_perms.iteritems():
+            if enabled and not getattr(user_perms, perm):
+                granted.send(sender=user, perm=perm, object=obj)
+            elif not enabled and getattr(user_perms, perm):
+                revoked.send(sender=user, perm=perm, object=obj)
+            setattr(user_perms, perm, enabled)
+        
+        user_perms.save()
+    
+    else:
+        # removing all perms.
+        revoke_all(user, obj)
 
     return perms
 
@@ -229,24 +266,31 @@ def set_group_perms(group, perms, obj):
     """
     Set group permissions to exactly the specified permissions.
     """
+    if perms:
+        model = obj.__class__
+        permissions = permission_map[model]
+        all_perms = dict((p, False) for p in get_model_perms(model))
+        for perm in perms:
+            all_perms[perm] = True
+    
+        try:
+            group_perms = permissions.objects.get(group=group, obj=obj)
+        except permissions.DoesNotExist:
+            group_perms = permissions(group=group, obj=obj)
+    
+        for perm, enabled in all_perms.iteritems():
+            if enabled and not getattr(group_perms, perm):
+                granted.send(sender=group, perm=perm, object=obj)
+            elif not enabled and getattr(group_perms, perm):
+                revoked.send(sender=group, perm=perm, object=obj)
+    
+            setattr(group_perms, perm, enabled)
+    
+        group_perms.save()
 
-    model = obj.__class__
-    permissions = permission_map[model]
-    all_perms = dict((p, False) for p in get_model_perms(model))
-    for perm in perms:
-        all_perms[perm] = True
-
-    group_perms, chaff = permissions.objects.get_or_create(group=group, obj=obj)
-
-    for perm, enabled in all_perms.iteritems():
-        if enabled and not getattr(group_perms, perm):
-            granted.send(sender=group, perm=perm, object=obj)
-        elif not enabled and getattr(group_perms, perm):
-            revoked.send(sender=group, perm=perm, object=obj)
-
-        setattr(group_perms, perm, enabled)
-
-    group_perms.save()
+    else:
+        # removing all perms.
+        revoke_all_group(group, obj)
 
     return perms
 
@@ -443,7 +487,7 @@ def group_has_perm(group, perm, obj):
      * The model is not registered for permissions
      * The permission does not exist on this model
     """
-    
+
     model = obj.__class__
     try:
         permissions = permission_map[model]
@@ -461,6 +505,34 @@ def group_has_perm(group, perm, obj):
     return permissions.objects.filter(group=group, obj=obj, **d).exists()
 
 
+def user_has_any_perms(user, obj):
+    """
+    Check whether the User has *any* permission on the given object.
+    """
+
+    model = obj.__class__
+    try:
+        permissions = permission_map[model]
+    except KeyError:
+        return False
+
+    return permissions.objects.filter(user=user, obj=obj).exists()
+
+
+def group_has_any_perms(group, obj):
+    """
+    Check whether the Group has *any* permission on the given object.
+    """
+
+    model = obj.__class__
+    try:
+        permissions = permission_map[model]
+    except KeyError:
+        return False
+
+    return permissions.objects.filter(group=group, obj=obj).exists()
+
+
 def get_users(obj):
     """
     Retrieve the list of Users that have permissions on the given object.
