@@ -1,6 +1,6 @@
 import os
 
-from django.conf.urls.defaults import *
+from django.conf.urls.defaults import url, patterns
 
 urlpatterns = patterns('object_permissions.views.groups',
     # Groups
@@ -18,4 +18,4 @@ root = '%s/media' % os.path.dirname(os.path.realpath(__file__))
 urlpatterns += patterns('',
     (r'^object_permissions_media/(?P<path>.*)', 'django.views.static.serve',\
      {'document_root':  root}),
-)
\ No newline at end of file
+)
